@@ -33,7 +33,7 @@
 
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
-#include <libxfce4panel/xfce-panel-plugin.h>
+#include <libxfce4panel/libxfce4panel.h>
 #include <libxfce4panel/xfce-panel-convenience.h>
 
 #include <stdint.h>
@@ -47,9 +47,12 @@ typedef struct param_t {
     /* Configurable parameters */
     char           *acCmd; /* Commandline to spawn */
     int             fTitleDisplayed;
+    int             fTitleDisplayedtmp;
     char           *acTitle;
     uint32_t        iPeriod_ms;
+    uint32_t        iPeriod_mstmp;
     char           *acFont;
+    char           *acFonttmp;
 } param_t;
 
 typedef struct conf_t {
@@ -73,14 +76,16 @@ typedef struct monitor_t {
     GtkWidget      *wImgButton;
     char           *onClickCmd;
     char           *onValClickCmd;
+    int             iconused;
+    char           *iconName;
 } monitor_t;
 
 typedef struct genmon_t {
-    XfcePanelPlugin *plugin;
-    unsigned int    iTimerId; /* Cyclic update */
-    struct conf_t   oConf;
+    XfcePanelPlugin    *plugin;
+    unsigned int        iTimerId; /* Cyclic update */
+    struct conf_t       oConf;
     struct monitor_t    oMonitor;
-    char            *acValue; /* Commandline resulting string */
+    char               *acValue; /* Commandline resulting string */
 } genmon_t;
 
 /**************************************************************/
@@ -91,6 +96,8 @@ static void ExecOnClickCmd (GtkWidget *p_wSc, void *p_pvPlugin)
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
     GError *error = NULL;
 
+    DBG("\n");
+
     xfce_spawn_command_line_on_screen( gdk_screen_get_default(), poMonitor->onClickCmd, 0, 0, &error );
     if (error) {
         char *first = g_strdup_printf (_("Could not run \"%s\""), poMonitor->onClickCmd);
@@ -111,6 +118,8 @@ static void ExecOnValClickCmd (GtkWidget *p_wSc, void *p_pvPlugin)
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
     GError *error = NULL;
 
+    DBG("\n");
+
     xfce_spawn_command_line_on_screen( gdk_screen_get_default(), poMonitor->onValClickCmd, 0, 0, &error );
     if (error) {
         char *first = g_strdup_printf (_("Could not run \"%s\""), poMonitor->onValClickCmd);
@@ -135,6 +144,10 @@ static int DisplayCmdOutput (struct genmon_t *p_poPlugin)
     char  *end;
     int    newVersion=0;
 
+    poMonitor->iconused=0;
+
+    DBG("\n");
+
     g_free (p_poPlugin->acValue);
     if (poConf->acCmd[0])
         p_poPlugin->acValue = genmon_SpawnCmd (poConf->acCmd, 1);
@@ -185,6 +198,80 @@ static int DisplayCmdOutput (struct genmon_t *p_poPlugin)
         gtk_widget_hide (poMonitor->wImage);
     }
 
+    /* Test if the result is an Icon */
+    begin=strstr(p_poPlugin->acValue, "<icon>");
+    end=strstr(p_poPlugin->acValue, "</icon>");
+    if (begin && end && begin < end)
+    {
+        gint size;
+        gint icon_size;
+        poMonitor->iconused = 1;
+
+        /* Get the icon name */
+        poMonitor->iconName = g_strndup (begin + 6, end - begin - 6);
+
+        #if !LIBXFCE4PANEL_CHECK_VERSION (4, 13, 0)
+          GtkStyleContext *context;
+          GtkBorder padding, border;
+          gint width;
+          gint xthickness;
+          gint ythickness;
+        #endif
+          size = xfce_panel_plugin_get_size (p_poPlugin->plugin) / xfce_panel_plugin_get_nrows (p_poPlugin->plugin);
+          gtk_widget_set_size_request (GTK_WIDGET (poMonitor->wButton), size, size);
+        #if LIBXFCE4PANEL_CHECK_VERSION (4,13,0)
+          icon_size = xfce_panel_plugin_get_icon_size (XFCE_PANEL_PLUGIN (p_poPlugin->plugin));
+        #else
+          /* Calculate the size of the widget because the theme can override it */
+          context = gtk_widget_get_style_context (GTK_WIDGET (poMonitor->wButton));
+          gtk_style_context_get_padding (context, gtk_widget_get_state_flags (GTK_WIDGET (poMonitor->wButton)), &padding);
+          gtk_style_context_get_border (context, gtk_widget_get_state_flags (GTK_WIDGET (poMonitor->wButton)), &border);
+          xthickness = padding.left + padding.right + border.left + border.right;
+          ythickness = padding.top + padding.bottom + border.top + border.bottom;
+
+          /* Calculate the size of the space left for the icon */
+          width = size - 2 * MAX (xthickness, ythickness);
+
+          /* Since symbolic icons are usually only provided in 16px we
+           * try to be clever and use size steps */
+          if (width <= 21)
+            icon_size = 16;
+          else if (width >=22 && width <= 29)
+            icon_size = 24;
+          else if (width >= 30 && width <= 40)
+            icon_size = 32;
+          else
+            icon_size = width;
+        #endif
+
+          gtk_image_set_from_icon_name (GTK_IMAGE (poMonitor->wImage), poMonitor->iconName, icon_size);
+          gtk_image_set_pixel_size (GTK_IMAGE (poMonitor->wImage), icon_size);  
+          gtk_image_set_from_icon_name (GTK_IMAGE (poMonitor->wImgButton), poMonitor->iconName, icon_size);
+          gtk_image_set_pixel_size (GTK_IMAGE (poMonitor->wImgButton), icon_size); 
+
+        /* Test if the result has a clickable Icon (button) */
+        begin=strstr(p_poPlugin->acValue, "<iconclick>");
+        end=strstr(p_poPlugin->acValue, "</iconclick>");
+        if (begin && end && begin < end)
+        {
+            /* Get the command path */
+            g_free (poMonitor->onClickCmd);
+            poMonitor->onClickCmd = g_strndup (begin + 11, end - begin - 11);
+
+            gtk_widget_show (poMonitor->wButton);
+            gtk_widget_show (poMonitor->wImgButton);
+            gtk_widget_hide (poMonitor->wImage);
+        }
+        else
+        {
+            gtk_widget_hide (poMonitor->wButton);
+            gtk_widget_hide (poMonitor->wImgButton);
+            gtk_widget_show (poMonitor->wImage);
+        }
+
+        newVersion=1;
+    }
+
     /* Test if the result is a Text */
     begin=strstr(p_poPlugin->acValue, "<txt>");
     end=strstr(p_poPlugin->acValue, "</txt>");
@@ -289,6 +376,8 @@ static gboolean SetTimer (void *p_pvPlugin)
     struct genmon_t *poPlugin = (genmon_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
 
+    DBG("\n");
+
     DisplayCmdOutput (poPlugin);
 
     if (poPlugin->iTimerId == 0)
@@ -327,12 +416,15 @@ static genmon_t *genmon_create_control (XfcePanelPlugin *plugin)
 
     poPlugin->plugin = plugin;
 
+    DBG("\n");
+
     poConf->acCmd = g_strdup ("");
     poConf->acTitle = g_strdup ("(genmon)");
 
-    poConf->fTitleDisplayed = 1;
+    poConf->fTitleDisplayedtmp = poConf->fTitleDisplayed = 1;
 
     poConf->iPeriod_ms = 30 * 1000;
+    poConf->iPeriod_mstmp = 30 * 1000;
     poPlugin->iTimerId = 0;
 
     // PangoFontDescription needs a font and we can't use "(Default)" anymore.
@@ -396,7 +488,7 @@ static genmon_t *genmon_create_control (XfcePanelPlugin *plugin)
         GTK_WIDGET (poMonitor->wImage), TRUE, FALSE, 0);
 
     /* Add Button */
-    poMonitor->wButton = xfce_create_panel_button ();
+    poMonitor->wButton = xfce_panel_create_button ();
     #if GTK_CHECK_VERSION (3, 16, 0)
          context = gtk_widget_get_style_context(poMonitor->wButton);
          gtk_style_context_add_class(context,"genmon_imagebutton");
@@ -421,7 +513,7 @@ static genmon_t *genmon_create_control (XfcePanelPlugin *plugin)
         GTK_WIDGET (poMonitor->wValue), TRUE, FALSE, 0);
 
     /* Add Value Button */
-    poMonitor->wValButton = xfce_create_panel_button ();
+    poMonitor->wValButton = xfce_panel_create_button ();
     #if GTK_CHECK_VERSION (3, 16, 0)
          context = gtk_widget_get_style_context(poMonitor->wValButton);
          gtk_style_context_add_class(context,"genmon_valuebutton");
@@ -509,6 +601,7 @@ static void genmon_free (XfcePanelPlugin *plugin, genmon_t *poPlugin)
 /* Plugin API */
 {
     TRACE ("genmon_free()\n");
+    DBG("\n");
 
     if (poPlugin->iTimerId)
         g_source_remove (poPlugin->iTimerId);
@@ -516,6 +609,7 @@ static void genmon_free (XfcePanelPlugin *plugin, genmon_t *poPlugin)
     g_free (poPlugin->oConf.oParam.acCmd);
     g_free (poPlugin->oConf.oParam.acTitle);
     g_free (poPlugin->oConf.oParam.acFont);
+    g_free (poPlugin->oConf.oParam.acFonttmp);
     g_free (poPlugin->oMonitor.onClickCmd);
     g_free (poPlugin->acValue);
     g_free (poPlugin);
@@ -551,8 +645,8 @@ static int SetMonitorFont (void *p_pvPlugin)
     css = g_strdup_printf(".label { font: %s; }", 
 #endif
                                     poConf->acFont);                        
-    /* Setup Gtk style */
-    DBG("css: %s",css);
+
+    DBG("\n");
     
     css_provider = gtk_css_provider_new ();
     gtk_css_provider_load_from_data (css_provider, css, strlen(css), NULL);
@@ -612,6 +706,8 @@ static void genmon_read_config (XfcePanelPlugin *plugin, genmon_t *poPlugin)
     char           *file;
     XfceRc         *rc;
 
+    DBG("\n");
+
     if (!(file = xfce_panel_plugin_lookup_rc_file (plugin)))
         return;
 
@@ -660,6 +756,8 @@ static void genmon_write_config (XfcePanelPlugin *plugin, genmon_t *poPlugin)
     XfceRc *rc;
     char *file;
 
+    DBG("\n");
+
     if (!(file = xfce_panel_plugin_save_location (plugin, TRUE)))
         return;
 
@@ -694,6 +792,8 @@ static void SetCmd (GtkWidget *p_wTF, void *p_pvPlugin)
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     const char     *pcCmd = gtk_entry_get_text (GTK_ENTRY (p_wTF));
 
+    DBG("\n");
+
     g_free (poConf->acCmd);
     poConf->acCmd = g_strdup (pcCmd);
 }/* SetCmd() */
@@ -706,16 +806,14 @@ static void ToggleTitle (GtkWidget *p_w, void *p_pvPlugin)
     struct genmon_t *poPlugin = (genmon_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct gui_t   *poGUI = &(poPlugin->oConf.oGUI);
-    struct monitor_t *poMonitor = &(poPlugin->oMonitor);
 
-    poConf->fTitleDisplayed =
+    DBG("\n");
+    
+    poConf->fTitleDisplayedtmp =
         gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (p_w));
     gtk_widget_set_sensitive (GTK_WIDGET (poGUI->wTF_Title),
-        poConf->fTitleDisplayed);
-    if (poConf->fTitleDisplayed)
-        gtk_widget_show (GTK_WIDGET (poMonitor->wTitle));
-    else
-        gtk_widget_hide (GTK_WIDGET (poMonitor->wTitle));
+        poConf->fTitleDisplayedtmp);
+
 }/* ToggleTitle() */
 
 /**************************************************************/
@@ -727,6 +825,8 @@ static void SetLabel (GtkWidget *p_wTF, void *p_pvPlugin)
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
     const char     *acTitle = gtk_entry_get_text (GTK_ENTRY (p_wTF));
+
+    DBG("\n");
     
     g_free (poConf->acTitle);
     poConf->acTitle = g_strdup (acTitle);
@@ -743,8 +843,10 @@ static void SetPeriod (GtkWidget *p_wSc, void *p_pvPlugin)
     float           r;
 
     TRACE ("SetPeriod()\n");
+    DBG("\n");
+
     r = gtk_spin_button_get_value (GTK_SPIN_BUTTON (p_wSc));
-    poConf->iPeriod_ms = (r * 1000);
+    poConf->iPeriod_mstmp = (r * 1000);
 }/* SetPeriod() */
 
 /**************************************************************/
@@ -757,6 +859,8 @@ static void UpdateConf (void *p_pvPlugin)
     struct gui_t   *poGUI = &(poConf->oGUI);
 
     TRACE ("UpdateConf()\n");
+    DBG("\n");
+
     SetCmd (poGUI->wTF_Cmd, poPlugin);
     SetLabel (poGUI->wTF_Title, poPlugin);
     SetMonitorFont (poPlugin);
@@ -782,6 +886,8 @@ static void About (XfcePanelPlugin *plugin)
       NULL
     };
 
+  DBG("\n");
+
   icon = xfce_panel_pixbuf_from_source ("utilities-system-monitor", NULL, 32);
   gtk_show_about_dialog (NULL,
                          "logo",         icon,
@@ -789,7 +895,7 @@ static void About (XfcePanelPlugin *plugin)
                          "version",      VERSION,
                          "program-name", PACKAGE,
                          "comments",     _("Cyclically spawns a script/program, captures its output and displays the resulting string in the panel"),
-                         "website",      "http://goodies.xfce.org/projects/panel-plugins/xfce4-genmon-plugin",
+                         "website",      "https://docs.xfce.org/panel-plugins/xfce4-genmon-plugin",
                          "copyright",    _("Copyright \xc2\xa9 2004 Roger Seguin\nCopyright \xc2\xa9 2006 Julien Devemy\nCopyright \xc2\xa9 2016 Tony Paulic\n"),
                          "authors",      auth,
                          NULL);
@@ -808,6 +914,8 @@ static void ChooseFont (GtkWidget *p_wPB, void *p_pvPlugin)
     const char     *pcFont;
     int             iResponse;
 
+    DBG("\n");
+
     wDialog = gtk_font_chooser_dialog_new (_("Font Selection"),
         GTK_WINDOW(gtk_widget_get_toplevel(p_wPB)));
     gtk_window_set_transient_for (GTK_WINDOW (wDialog),
@@ -818,9 +926,9 @@ static void ChooseFont (GtkWidget *p_wPB, void *p_pvPlugin)
     if (iResponse == GTK_RESPONSE_OK) {
         pcFont = gtk_font_chooser_get_font (GTK_FONT_CHOOSER (wDialog));
         if (pcFont) {
-            g_free (poConf->acFont);
-            poConf->acFont = g_strdup (pcFont);
-            gtk_button_set_label (GTK_BUTTON (p_wPB), poConf->acFont);
+            g_free (poConf->acFonttmp);
+            poConf->acFonttmp = g_strdup (pcFont);
+            gtk_button_set_label (GTK_BUTTON (p_wPB), poConf->acFonttmp);
         }
     }
     gtk_widget_destroy (wDialog);
@@ -831,14 +939,43 @@ static void ChooseFont (GtkWidget *p_wPB, void *p_pvPlugin)
 static void genmon_dialog_response (GtkWidget *dlg, int response,
     genmon_t *genmon)
 {
-    UpdateConf (genmon);
-    gtk_widget_destroy (dlg);
-    xfce_panel_plugin_unblock_menu (genmon->plugin);
-    genmon_write_config (genmon->plugin, genmon);
-    /* Do not wait the next timer to update display */
-    DisplayCmdOutput (genmon);
+	struct param_t *poConf = &(genmon->oConf.oParam);
+    struct monitor_t *poMonitor = &(genmon->oMonitor);
+
+    DBG("\n");
+	
+	if (response == GTK_RESPONSE_OK) {
+		if (poConf->acFonttmp)
+		{
+			g_free (poConf->acFont);
+		 	poConf->acFont = g_strdup (poConf->acFonttmp);
+		}
+		
+	 	poConf->fTitleDisplayed = poConf->fTitleDisplayedtmp;
+	 	if (poConf->fTitleDisplayed)
+			gtk_widget_show (GTK_WIDGET (poMonitor->wTitle));
+		else
+			gtk_widget_hide (GTK_WIDGET (poMonitor->wTitle));
+			
+		poConf->iPeriod_ms = poConf->iPeriod_mstmp;
+						
+		UpdateConf (genmon);
+		genmon_write_config (genmon->plugin, genmon);
+		/* Do not wait the next timer to update display */
+		DisplayCmdOutput (genmon);
+	}
+	else {
+		poConf->acFonttmp = g_strdup (poConf->acFont);
+		poConf->fTitleDisplayedtmp = poConf->fTitleDisplayed;
+		poConf->iPeriod_mstmp = poConf->iPeriod_ms;
+	}
+	
+	gtk_widget_destroy (dlg);
+	xfce_panel_plugin_unblock_menu (genmon->plugin);
 }
 
+/**************************************************************/
+
 static void genmon_create_options (XfcePanelPlugin *plugin,
     genmon_t *poPlugin)
 /* Plugin API */
@@ -849,14 +986,25 @@ static void genmon_create_options (XfcePanelPlugin *plugin,
     struct gui_t   *poGUI = &(poPlugin->oConf.oGUI);
 
     TRACE ("genmon_create_options()\n");
+    DBG("\n");
 
     xfce_panel_plugin_block_menu (plugin);
+    poConf->fTitleDisplayedtmp = poConf->fTitleDisplayed;
+    poConf->iPeriod_mstmp = poConf->iPeriod_ms;
 
+#if LIBXFCE4UI_CHECK_VERSION (4, 15, 1)
+    dlg = xfce_titled_dialog_new_with_mixed_buttons (_("Generic Monitor"),
+         GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
+         GTK_DIALOG_DESTROY_WITH_PARENT,
+         "gtk-save", _("Save"), GTK_RESPONSE_OK,
+         NULL);
+#else
     dlg = xfce_titled_dialog_new_with_buttons (_("Generic Monitor"),
         GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
         GTK_DIALOG_DESTROY_WITH_PARENT,
-        "gtk-close", GTK_RESPONSE_OK,
+        "gtk-save", GTK_RESPONSE_OK,
         NULL);
+#endif
 
     gtk_window_set_resizable (GTK_WINDOW (dlg), FALSE);
     gtk_window_set_icon_name (GTK_WINDOW (dlg), "utilities-system-monitor");
@@ -915,6 +1063,8 @@ static void genmon_set_orientation (XfcePanelPlugin *plugin,
 {
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
 
+    DBG("\n");
+
     gtk_orientable_set_orientation(GTK_ORIENTABLE(poMonitor->wBox), p_iOrientation);
     gtk_orientable_set_orientation(GTK_ORIENTABLE(poMonitor->wImgBox), p_iOrientation);
 
@@ -938,15 +1088,65 @@ static gboolean genmon_set_size (XfcePanelPlugin *plugin, int size, genmon_t *po
 {
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
 
-    if (xfce_panel_plugin_get_orientation (plugin) == GTK_ORIENTATION_HORIZONTAL)
+    if (poMonitor->iconused)
     {
-        if (size>BORDER)
-            gtk_widget_set_size_request(GTK_WIDGET(poMonitor->wBar),8, size-BORDER*2);
+
+    #if !LIBXFCE4PANEL_CHECK_VERSION (4, 13, 0)
+        GtkStyleContext *context;
+        GtkBorder padding, border;
+        gint width;
+        gint xthickness;
+        gint ythickness;
+    #endif
+        gint icon_size;
+        size /= xfce_panel_plugin_get_nrows (plugin);
+        gtk_widget_set_size_request (GTK_WIDGET (poMonitor->wButton), size, size);
+
+    #if LIBXFCE4PANEL_CHECK_VERSION (4,13,0)
+        icon_size = xfce_panel_plugin_get_icon_size (XFCE_PANEL_PLUGIN (plugin));
+    #else
+        /* Calculate the size of the widget because the theme can override it */
+        context = gtk_widget_get_style_context (GTK_WIDGET (poMonitor->wButton));
+        gtk_style_context_get_padding (context, gtk_widget_get_state_flags (GTK_WIDGET (poMonitor->wButton)), &padding);
+        gtk_style_context_get_border (context, gtk_widget_get_state_flags (GTK_WIDGET (poMonitor->wButton)), &border);
+        xthickness = padding.left + padding.right + border.left + border.right;
+        ythickness = padding.top + padding.bottom + border.top + border.bottom;
+
+        /* Calculate the size of the space left for the icon */
+        width = size - 2 * MAX (xthickness, ythickness);
+
+        /* Since symbolic icons are usually only provided in 16px we
+        * try to be clever and use size steps */
+        if (width <= 21)
+            icon_size = 16;
+        else if (width >=22 && width <= 29)
+            icon_size = 24;
+        else if (width >= 30 && width <= 40)
+            icon_size = 32;
+        else
+            icon_size = width;
+    #endif
+
+    DBG("\n");
+
+        gtk_image_set_from_icon_name (GTK_IMAGE (poMonitor->wImage), poMonitor->iconName, icon_size);
+        gtk_image_set_pixel_size (GTK_IMAGE (poMonitor->wImage), icon_size);  
+        gtk_image_set_from_icon_name (GTK_IMAGE (poMonitor->wImgButton), poMonitor->iconName, icon_size);
+        gtk_image_set_pixel_size (GTK_IMAGE (poMonitor->wImgButton), icon_size);
+
     }
     else
     {
-        if (size>BORDER)
-            gtk_widget_set_size_request(GTK_WIDGET(poMonitor->wBar), size-BORDER*2, 8);
+       if (xfce_panel_plugin_get_orientation (plugin) == GTK_ORIENTATION_HORIZONTAL)
+       {
+           if (size>BORDER)
+               gtk_widget_set_size_request(GTK_WIDGET(poMonitor->wBar),8, size-BORDER*2);
+       }
+       else
+       {
+           if (size>BORDER)
+               gtk_widget_set_size_request(GTK_WIDGET(poMonitor->wBar), size-BORDER*2, 8);
+       }
     }
 
     return TRUE;
@@ -961,6 +1161,8 @@ static gboolean genmon_remote_event (XfcePanelPlugin *plugin,
                                     const GValue *value,
                                     genmon_t *genmon)
 {
+    DBG("\n");
+
     g_return_val_if_fail (value == NULL || G_IS_VALUE (value), FALSE);
     if (strcmp (name, "refresh") == 0)
         {
@@ -984,6 +1186,8 @@ static void genmon_construct (XfcePanelPlugin *plugin)
 {
     genmon_t *genmon;
 
+    DBG("\n");
+
     xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
     genmon = genmon_create_control (plugin);
@@ -1020,6 +1224,8 @@ static void genmon_construct (XfcePanelPlugin *plugin)
 
     g_signal_connect (G_OBJECT (genmon->oMonitor.wValButton), "clicked",
         G_CALLBACK (ExecOnValClickCmd), genmon);        
+
+    xfce_panel_plugin_set_small (plugin, TRUE);
 }
 
 XFCE_PANEL_PLUGIN_REGISTER (genmon_construct)
